from ecsclient.common.configuration import certificate, configuration_properties, licensing, feature

certificate = certificate
configuration_properties = configuration_properties
licensing = licensing
feature = feature
