from ecsclient.common.provisioning import base_url, bucket, data_store, storage_pool, virtual_data_center, vdc_keystore

base_url = base_url
bucket = bucket
data_store = data_store
storage_pool = storage_pool
virtual_data_center = virtual_data_center
vdc_keystore = vdc_keystore
